import _ from "lodash";

export class SmartDataDS {

    static get GET() { return '/get.php'; }
    static get HC()  { return '/health_check.php'; }


    constructor(instanceSettings, $q, backendSrv, templateSrv) {
        this.url = instanceSettings.url;
        this.q = $q;
        this.backendSrv = backendSrv;
    }

    query(options) {

        //console.log("[SmartDataDS::query() => options]", options);

        var t0 = options.range.from.valueOf();
        var t1 = options.range.to.valueOf();
        var period = options.intervalMs;


        var get_options = {
            method: 'POST',
            headers : {'Content-Type': 'application/json'},
            url: this.url + SmartDataDS.GET,
            data: null
        };

        //console.log("[SmartDataDS::query() => get_options]", get_options);

        var targets_response = [];

        for(let target_index=0, len = options.targets.length; target_index < len; target_index++) {

            if (options.targets[target_index].hide)
                continue;

            console.log("[SmartDataDS::query() => options.targets[i]]", options.targets[target_index]);

            var request_parameters = {
                'series' : {
                    //'version': parseInt(options.targets[target_index].version),
                    'version' : '1.1',
                    'unit': parseInt(options.targets[target_index].unit),
                    'x': parseInt(options.targets[target_index].x),
                    'y': parseInt(options.targets[target_index].y),
                    'z': parseInt(options.targets[target_index].z),
                    'r': parseInt(options.targets[target_index].r),
                    't0': parseInt(t0 * 1000),
                    't1': parseInt(t1 * 1000),
                    'dev': parseInt(options.targets[target_index].dev),
                    'workflow': parseInt(options.targets[target_index].workflow),
                },
                'credentials' : {
                    'domain' : options.targets[target_index].domain
                }
            };

            if (options.targets[target_index].username.length)
                request_parameters['credentials']['username'] = options.targets[target_index].username
            if (options.targets[target_index].password.length)
                request_parameters['credentials']['password'] = options.targets[target_index].password

            if (options.targets[target_index].type !== '') {
                request_parameters['options'] = new Object();
                request_parameters['options']['type'] = options.targets[target_index].type;
            }

            if ('aggregator' in options.targets[target_index] && options.targets[target_index].aggregator.length) {
                request_parameters['aggregator'] = new Object();
                request_parameters['aggregator']['name'] = options.targets[target_index].aggregator;
            }
            if ('aggregatorRange' in options.targets[target_index] && options.targets[target_index].aggregatorRange.length && options.targets[target_index].aggregator.length)
                request_parameters['aggregator']['range'] = parseInt(options.targets[target_index].aggregatorRange);
            if ('aggregatorParameter' in options.targets[target_index] && options.targets[target_index].aggregatorParameter.length && options.targets[target_index].aggregator.length)
                request_parameters['aggregator']['parameter'] = parseFloat(options.targets[target_index].aggregatorParameter);

            //add here the json
            if ('aggregatorDelay' in options.targets[target_index] && options.targets[target_index].aggregatorDelay.length && options.targets[target_index].aggregator.length)
                request_parameters['aggregator']['delay'] = parseInt(options.targets[target_index].aggregatorDelay);
            if ('aggregatorSpacing' in options.targets[target_index] && options.targets[target_index].aggregatorSpacing.length && options.targets[target_index].aggregator.length)
                request_parameters['aggregator']['spacing'] = parseFloat(options.targets[target_index].aggregatorSpacing);

            if ('aggregatorExtra' in options.targets[target_index] && options.targets[target_index].aggregatorExtra.length && options.targets[target_index].aggregator.length) {
                var tmp = (String(options.targets[target_index].aggregatorExtra)).split(",");
                var t;
                for (t = 0; t < tmp.length; t++) {
                    var aux = (tmp[t]).split(":");
                    request_parameters['aggregator'][aux[0]] = parseFloat(aux[1])
                }
            }
            // if ('aggregatorDrift' in options.targets[target_index] && options.targets[target_index].aggregatorDrift.length && options.targets[target_index].aggregator.length)
            //     request_parameters['aggregator']['drift'] = parseFloat(options.targets[target_index].aggregatorDrift)
            // if ('aggregatorStuck' in options.targets[target_index] && options.targets[target_index].aggregatorStuck.length && options.targets[target_index].aggregator.length)
            //     request_parameters['aggregator']['stuck'] = parseFloat(options.targets[target_index].aggregatorStuck)
            // if ('aggregatorBias' in options.targets[target_index] && options.targets[target_index].aggregatorBias.length && options.targets[target_index].aggregator.length)
            //     request_parameters['aggregator']['bias'] = parseFloat(options.targets[target_index].aggregatorBias)
            // if ('aggregatorGain' in options.targets[target_index] && options.targets[target_index].aggregatorGain.length && options.targets[target_index].aggregator.length)
            //     request_parameters['aggregator']['gain'] = parseFloat(options.targets[target_index].aggregatorGain)
            get_options.data = request_parameters;

            var $q = this.q;

            var t_response = this.backendSrv.datasourceRequest(get_options).then(function (response, _q=$q) {
                var dfr = _q.defer();

                var data = [];

                _.each(response.data.series, function (r) {
                    var timestamp = parseInt(r.timestamp/1000);
                    var value = r.value;

                    if(timestamp >= t0 && timestamp <= t1){
                        data.push( [value, timestamp] );
                    }
                });

                //data.sort( function(a, b) { return a[1] - b[1]; } );

                var target = { "target": options.targets[target_index].alias, "datapoints": data }

                //console.log("[SmartDataDS::query() => target]", target);

                dfr.resolve(target);

                return dfr.promise;
            });

            targets_response.push(t_response);
        }


        var series = { "data": [] };

        return this.q.all( targets_response ).then(function(data) {
            //console.log("[SmartDataDS::query() => targets_response>>data]", data);

            for(let target_index=0, len = data.length; target_index < len; target_index++) {
                var target = data[target_index];
                series.data.push(target);
            }

            //console.log("[SmartDataDS::query() => targets_response>>series]", series);

            return series;
        });
    }

    testDatasource() {
        console.log("[SmartDataDS::testDatasource()]");

        return this.backendSrv.datasourceRequest({
            url: this.url + SmartDataDS.HC,
            method: 'GET'
        }).then(response => {
            if (response.status === 204) {
                //console.log("[SmartDataDS::testDatasource()] Sucess");
                return { status: "success", message: "Data source is working!", title: "Success" };
            }
        });
    }

    annotationQuery(options) {
        /*var query = this.templateSrv.replace(options.annotation.query, {}, 'glob');
        var annotationQuery = {
            range: options.range,
            annotation: {
                name: options.annotation.name,
                datasource: options.annotation.datasource,
                enable: options.annotation.enable,
                iconColor: options.annotation.iconColor,
                query: query
            },
            rangeRaw: options.rangeRaw
        };

        return this.doRequest({
            url: this.url + '/annotations',
            method: 'POST',
            data: annotationQuery
        }).then(result => {
            return result.data;
        });*/
    }

    /*
    metricFindQuery(query) {
        var interpolated = {
            target: this.templateSrv.replace(query, null, 'regex')
        };

        return this.doRequest({
            url: this.url + '/search',
            data: interpolated,
            method: 'POST',
        }).then(this.mapToTextValue);
    }
    */


    metricFindQuery(query) {
        return [ { "text" : query, "value": query} ];
    }

    /*
    mapToTextValue(result) {
        return _.map(result.data, (d, i) => {
            if (d && d.text && d.value) {
                return { text: d.text, value: d.value };
            } else if (_.isObject(d)) {
                return { text: d, value: i};
            }
            return { text: d, value: d };
        });
    }
    */

    getDefaultAlias(target) {
        /*if (!target.metric) {
        return "";
        }*/

        var interestAlias = "{ unit:"+target.unit+
                            ", xyz:{"+target.x+","+target.y+","+target.z+"}"+
                            ", r:"+target.r+" }";

        return interestAlias;
    };

    getUnits(target) {
        // Needs to be reimplement
        return [
            { "text" : "Length"              , "value" : "2224441636" },
            { "text" : "Mass"                , "value" : "2224212260" },
            { "text" : "Time"                , "value" : "2224183588" },
            { "text" : "Current"             , "value" : "2224180004" },
            { "text" : "Electric Current"    , "value" : "2224180004" },
            { "text" : "Temperature"         , "value" : "2224179556" },
            { "text" : "Amount of Substance" , "value" : "2224179500" },
            { "text" : "Luminous Intensity"  , "value" : "2224179493" },
            { "text" : "Area"                , "value" : "2224703780" },
            { "text" : "Volume"              , "value" : "2224965924" },
            { "text" : "Speed"               , "value" : "2224437540" },
            { "text" : "Velocity"            , "value" : "2224437540" },
            { "text" : "Acceleration"        , "value" : "2224433444" }
        ];
    }

    //getAggregators(target) {
    //    return [
    //        { "text" : "None"       , "value" : ""          },
    //        { "text" : "Min"        , "value" : "min"       },
    //        { "text" : "Mean"       , "value" : "mean"      },
    //        { "text" : "Max"        , "value" : "max"       },
    //        { "text" : "DBP"        , "value" : "dbp"       },
    //        { "text" : "Threshold"  , "value" : "threshold" },
    //        { "text" : "Last Value" , "value" : "lastValue" }
    //    ];
    //}

}
